/* This file was generated by syncqt. */
#ifndef QT_QTCONCURRENT_VERSION_H
#define QT_QTCONCURRENT_VERSION_H

#define QTCONCURRENT_VERSION_STR "5.15.19"

#define QTCONCURRENT_VERSION 0x050F13

#endif // QT_QTCONCURRENT_VERSION_H
