/* This file was generated by syncqt. */
#ifndef QT_QTVULKANSUPPORT_VERSION_H
#define QT_QTVULKANSUPPORT_VERSION_H

#define QTVULKANSUPPORT_VERSION_STR "5.15.19"

#define QTVULKANSUPPORT_VERSION 0x050F13

#endif // QT_QTVULKANSUPPORT_VERSION_H
