/* This file was generated by syncqt. */
#ifndef QT_QTNETWORK_VERSION_H
#define QT_QTNETWORK_VERSION_H

#define QTNETWORK_VERSION_STR "5.15.19"

#define QTNETWORK_VERSION 0x050F13

#endif // QT_QTNETWORK_VERSION_H
